from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.rail_spec_version import RailSpecVersion
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schema import Schema


T = TypeVar("T", bound="RailSpec")


@_attrs_define
class RailSpec:
    """A JSONified version of the user provided railspec.

    Attributes:
        output_schema (Schema):
        prompt (str):
        version (RailSpecVersion):  Default: RailSpecVersion.VALUE_0.
        input_schema (Union[Unset, Schema]):
        instructions (Union[Unset, str]):
    """

    output_schema: "Schema"
    prompt: str
    version: RailSpecVersion = RailSpecVersion.VALUE_0
    input_schema: Union[Unset, "Schema"] = UNSET
    instructions: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        output_schema = self.output_schema.to_dict()

        prompt = self.prompt

        version = self.version.value

        input_schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.input_schema, Unset):
            input_schema = self.input_schema.to_dict()

        instructions = self.instructions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "outputSchema": output_schema,
                "prompt": prompt,
                "version": version,
            }
        )
        if input_schema is not UNSET:
            field_dict["inputSchema"] = input_schema
        if instructions is not UNSET:
            field_dict["instructions"] = instructions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.schema import Schema

        d = src_dict.copy()
        output_schema = Schema.from_dict(d.pop("outputSchema"))

        prompt = d.pop("prompt")

        version = RailSpecVersion(d.pop("version"))

        _input_schema = d.pop("inputSchema", UNSET)
        input_schema: Union[Unset, Schema]
        if isinstance(_input_schema, Unset):
            input_schema = UNSET
        else:
            input_schema = Schema.from_dict(_input_schema)

        instructions = d.pop("instructions", UNSET)

        rail_spec = cls(
            output_schema=output_schema,
            prompt=prompt,
            version=version,
            input_schema=input_schema,
            instructions=instructions,
        )

        rail_spec.additional_properties = d
        return rail_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Projection")


@_attrs_define
class Projection:
    """
    Attributes:
        id (Union[Unset, str]):
        guard_id (Union[Unset, str]):
        validator_id (Union[Unset, str]):
        document_id (Union[Unset, str]):
        page_id (Union[Unset, str]):
        chunk_id (Union[Unset, str]):
        projection (Union[Unset, List[float]]):
    """

    id: Union[Unset, str] = UNSET
    guard_id: Union[Unset, str] = UNSET
    validator_id: Union[Unset, str] = UNSET
    document_id: Union[Unset, str] = UNSET
    page_id: Union[Unset, str] = UNSET
    chunk_id: Union[Unset, str] = UNSET
    projection: Union[Unset, List[float]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        guard_id = self.guard_id

        validator_id = self.validator_id

        document_id = self.document_id

        page_id = self.page_id

        chunk_id = self.chunk_id

        projection: Union[Unset, List[float]] = UNSET
        if not isinstance(self.projection, Unset):
            projection = self.projection

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if guard_id is not UNSET:
            field_dict["guardId"] = guard_id
        if validator_id is not UNSET:
            field_dict["validatorId"] = validator_id
        if document_id is not UNSET:
            field_dict["documentId"] = document_id
        if page_id is not UNSET:
            field_dict["pageId"] = page_id
        if chunk_id is not UNSET:
            field_dict["chunkId"] = chunk_id
        if projection is not UNSET:
            field_dict["projection"] = projection

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        guard_id = d.pop("guardId", UNSET)

        validator_id = d.pop("validatorId", UNSET)

        document_id = d.pop("documentId", UNSET)

        page_id = d.pop("pageId", UNSET)

        chunk_id = d.pop("chunkId", UNSET)

        projection = cast(List[float], d.pop("projection", UNSET))

        projection = cls(
            id=id,
            guard_id=guard_id,
            validator_id=validator_id,
            document_id=document_id,
            page_id=page_id,
            chunk_id=chunk_id,
            projection=projection,
        )

        projection.additional_properties = d
        return projection

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.guard_guard_config import GuardGuardConfig
    from ..models.rail_spec import RailSpec


T = TypeVar("T", bound="Guard")


@_attrs_define
class Guard:
    """
    Attributes:
        name (str): A unique name for this guard.
        railspec (RailSpec): A JSONified version of the user provided railspec.
        description (Union[Unset, str]): A description of this guard.
        num_reasks (Union[Unset, int]): The number of re-asks to perform during validation when a validation fails.
        guard_config (Union[Unset, GuardGuardConfig]): Addtional guard configuration attributes.
        llm_endpoint (Union[Unset, str]): The endpoint for the targetted LLM.
    """

    name: str
    railspec: "RailSpec"
    description: Union[Unset, str] = UNSET
    num_reasks: Union[Unset, int] = UNSET
    guard_config: Union[Unset, "GuardGuardConfig"] = UNSET
    llm_endpoint: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        railspec = self.railspec.to_dict()

        description = self.description

        num_reasks = self.num_reasks

        guard_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.guard_config, Unset):
            guard_config = self.guard_config.to_dict()

        llm_endpoint = self.llm_endpoint

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "railspec": railspec,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if num_reasks is not UNSET:
            field_dict["numReasks"] = num_reasks
        if guard_config is not UNSET:
            field_dict["guardConfig"] = guard_config
        if llm_endpoint is not UNSET:
            field_dict["llmEndpoint"] = llm_endpoint

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.guard_guard_config import GuardGuardConfig
        from ..models.rail_spec import RailSpec

        d = src_dict.copy()
        name = d.pop("name")

        railspec = RailSpec.from_dict(d.pop("railspec"))

        description = d.pop("description", UNSET)

        num_reasks = d.pop("numReasks", UNSET)

        _guard_config = d.pop("guardConfig", UNSET)
        guard_config: Union[Unset, GuardGuardConfig]
        if isinstance(_guard_config, Unset):
            guard_config = UNSET
        else:
            guard_config = GuardGuardConfig.from_dict(_guard_config)

        llm_endpoint = d.pop("llmEndpoint", UNSET)

        guard = cls(
            name=name,
            railspec=railspec,
            description=description,
            num_reasks=num_reasks,
            guard_config=guard_config,
            llm_endpoint=llm_endpoint,
        )

        guard.additional_properties = d
        return guard

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.http_error_fields import HttpErrorFields


T = TypeVar("T", bound="HttpError")


@_attrs_define
class HttpError:
    """
    Attributes:
        status (int): A valid http status code
        message (str): A message explaining the status
        cause (Union[Unset, str]): Used to describe the origin of an error if that original error has meaning to the
            client.  This field should add specificity to 'message'.
        fields (Union[Unset, HttpErrorFields]): Used to identify specific fields in a JSON body that caused the error.
            Typically only used for 4xx type responses.  The key should be the json path to the invalid property and the
            value should be a list of error messages specific to that property.
        context (Union[Unset, str]): Used to identify what part of the request caused the error for non-JSON payloads.
    """

    status: int
    message: str
    cause: Union[Unset, str] = UNSET
    fields: Union[Unset, "HttpErrorFields"] = UNSET
    context: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status

        message = self.message

        cause = self.cause

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = self.fields.to_dict()

        context = self.context

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "message": message,
            }
        )
        if cause is not UNSET:
            field_dict["cause"] = cause
        if fields is not UNSET:
            field_dict["fields"] = fields
        if context is not UNSET:
            field_dict["context"] = context

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.http_error_fields import HttpErrorFields

        d = src_dict.copy()
        status = d.pop("status")

        message = d.pop("message")

        cause = d.pop("cause", UNSET)

        _fields = d.pop("fields", UNSET)
        fields: Union[Unset, HttpErrorFields]
        if isinstance(_fields, Unset):
            fields = UNSET
        else:
            fields = HttpErrorFields.from_dict(_fields)

        context = d.pop("context", UNSET)

        http_error = cls(
            status=status,
            message=message,
            cause=cause,
            fields=fields,
            context=context,
        )

        http_error.additional_properties = d
        return http_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.any_object import AnyObject


T = TypeVar("T", bound="Reask")


@_attrs_define
class Reask:
    """
    Attributes:
        incorrect_value (Union['AnyObject', List[Union['AnyObject', List[Union['AnyObject', bool, float, int, str]],
            bool, float, int, str]], Unset, bool, float, int, str]):
        error_message (Union[Unset, str]):
        fix_value (Union['AnyObject', List[Union['AnyObject', List[Union['AnyObject', bool, float, int, str]], bool,
            float, int, str]], Unset, bool, float, int, str]):
        path (Union[Unset, List[Union['AnyObject', List[Union['AnyObject', List[Union['AnyObject', bool, float, int,
            str]], bool, float, int, str]], bool, float, int, str]]]):
    """

    incorrect_value: Union[
        "AnyObject",
        List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
        Unset,
        bool,
        float,
        int,
        str,
    ] = UNSET
    error_message: Union[Unset, str] = UNSET
    fix_value: Union[
        "AnyObject",
        List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
        Unset,
        bool,
        float,
        int,
        str,
    ] = UNSET
    path: Union[
        Unset,
        List[
            Union[
                "AnyObject",
                List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
                bool,
                float,
                int,
                str,
            ]
        ],
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.any_object import AnyObject

        incorrect_value: Union[
            Dict[str, Any],
            List[Union[Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str]],
            Unset,
            bool,
            float,
            int,
            str,
        ]
        if isinstance(self.incorrect_value, Unset):
            incorrect_value = UNSET
        elif isinstance(self.incorrect_value, list):
            incorrect_value = []
            for componentsschemas_any_array_item_data in self.incorrect_value:
                componentsschemas_any_array_item: Union[
                    Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str
                ]
                if isinstance(componentsschemas_any_array_item_data, AnyObject):
                    componentsschemas_any_array_item = componentsschemas_any_array_item_data.to_dict()
                elif isinstance(componentsschemas_any_array_item_data, list):
                    componentsschemas_any_array_item = []
                    for componentsschemas_any_array_item_type_2_item_data in componentsschemas_any_array_item_data:
                        componentsschemas_any_array_item_type_2_item: Union[Dict[str, Any], bool, float, int, str]
                        if isinstance(componentsschemas_any_array_item_type_2_item_data, AnyObject):
                            componentsschemas_any_array_item_type_2_item = (
                                componentsschemas_any_array_item_type_2_item_data.to_dict()
                            )
                        else:
                            componentsschemas_any_array_item_type_2_item = (
                                componentsschemas_any_array_item_type_2_item_data
                            )
                        componentsschemas_any_array_item.append(componentsschemas_any_array_item_type_2_item)

                else:
                    componentsschemas_any_array_item = componentsschemas_any_array_item_data
                incorrect_value.append(componentsschemas_any_array_item)

        elif isinstance(self.incorrect_value, AnyObject):
            incorrect_value = self.incorrect_value.to_dict()
        else:
            incorrect_value = self.incorrect_value

        error_message = self.error_message

        fix_value: Union[
            Dict[str, Any],
            List[Union[Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str]],
            Unset,
            bool,
            float,
            int,
            str,
        ]
        if isinstance(self.fix_value, Unset):
            fix_value = UNSET
        elif isinstance(self.fix_value, list):
            fix_value = []
            for componentsschemas_any_array_item_data in self.fix_value:
                componentsschemas_any_array_item: Union[
                    Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str
                ]
                if isinstance(componentsschemas_any_array_item_data, AnyObject):
                    componentsschemas_any_array_item = componentsschemas_any_array_item_data.to_dict()
                elif isinstance(componentsschemas_any_array_item_data, list):
                    componentsschemas_any_array_item = []
                    for componentsschemas_any_array_item_type_2_item_data in componentsschemas_any_array_item_data:
                        componentsschemas_any_array_item_type_2_item: Union[Dict[str, Any], bool, float, int, str]
                        if isinstance(componentsschemas_any_array_item_type_2_item_data, AnyObject):
                            componentsschemas_any_array_item_type_2_item = (
                                componentsschemas_any_array_item_type_2_item_data.to_dict()
                            )
                        else:
                            componentsschemas_any_array_item_type_2_item = (
                                componentsschemas_any_array_item_type_2_item_data
                            )
                        componentsschemas_any_array_item.append(componentsschemas_any_array_item_type_2_item)

                else:
                    componentsschemas_any_array_item = componentsschemas_any_array_item_data
                fix_value.append(componentsschemas_any_array_item)

        elif isinstance(self.fix_value, AnyObject):
            fix_value = self.fix_value.to_dict()
        else:
            fix_value = self.fix_value

        path: Union[
            Unset,
            List[
                Union[
                    Dict[str, Any],
                    List[
                        Union[Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str]
                    ],
                    bool,
                    float,
                    int,
                    str,
                ]
            ],
        ] = UNSET
        if not isinstance(self.path, Unset):
            path = []
            for path_item_data in self.path:
                path_item: Union[
                    Dict[str, Any],
                    List[
                        Union[Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str]
                    ],
                    bool,
                    float,
                    int,
                    str,
                ]
                if isinstance(path_item_data, list):
                    path_item = []
                    for componentsschemas_any_array_item_data in path_item_data:
                        componentsschemas_any_array_item: Union[
                            Dict[str, Any], List[Union[Dict[str, Any], bool, float, int, str]], bool, float, int, str
                        ]
                        if isinstance(componentsschemas_any_array_item_data, AnyObject):
                            componentsschemas_any_array_item = componentsschemas_any_array_item_data.to_dict()
                        elif isinstance(componentsschemas_any_array_item_data, list):
                            componentsschemas_any_array_item = []
                            for (
                                componentsschemas_any_array_item_type_2_item_data
                            ) in componentsschemas_any_array_item_data:
                                componentsschemas_any_array_item_type_2_item: Union[
                                    Dict[str, Any], bool, float, int, str
                                ]
                                if isinstance(componentsschemas_any_array_item_type_2_item_data, AnyObject):
                                    componentsschemas_any_array_item_type_2_item = (
                                        componentsschemas_any_array_item_type_2_item_data.to_dict()
                                    )
                                else:
                                    componentsschemas_any_array_item_type_2_item = (
                                        componentsschemas_any_array_item_type_2_item_data
                                    )
                                componentsschemas_any_array_item.append(componentsschemas_any_array_item_type_2_item)

                        else:
                            componentsschemas_any_array_item = componentsschemas_any_array_item_data
                        path_item.append(componentsschemas_any_array_item)

                elif isinstance(path_item_data, AnyObject):
                    path_item = path_item_data.to_dict()
                else:
                    path_item = path_item_data
                path.append(path_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if incorrect_value is not UNSET:
            field_dict["incorrect_value"] = incorrect_value
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if fix_value is not UNSET:
            field_dict["fix_value"] = fix_value
        if path is not UNSET:
            field_dict["path"] = path

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.any_object import AnyObject

        d = src_dict.copy()

        def _parse_incorrect_value(
            data: object,
        ) -> Union[
            "AnyObject",
            List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
            Unset,
            bool,
            float,
            int,
            str,
        ]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemas_any_type_0 = []
                _componentsschemas_any_type_0 = data
                for componentsschemas_any_array_item_data in _componentsschemas_any_type_0:

                    def _parse_componentsschemas_any_array_item(
                        data: object,
                    ) -> Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            componentsschemas_any_array_item_type_0 = AnyObject.from_dict(data)

                            return componentsschemas_any_array_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, list):
                                raise TypeError()
                            componentsschemas_any_array_item_type_2 = []
                            _componentsschemas_any_array_item_type_2 = data
                            for (
                                componentsschemas_any_array_item_type_2_item_data
                            ) in _componentsschemas_any_array_item_type_2:

                                def _parse_componentsschemas_any_array_item_type_2_item(
                                    data: object,
                                ) -> Union["AnyObject", bool, float, int, str]:
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        componentsschemas_any_array_item_type_2_item_type_0 = AnyObject.from_dict(data)

                                        return componentsschemas_any_array_item_type_2_item_type_0
                                    except:  # noqa: E722
                                        pass
                                    return cast(Union["AnyObject", bool, float, int, str], data)

                                componentsschemas_any_array_item_type_2_item = (
                                    _parse_componentsschemas_any_array_item_type_2_item(
                                        componentsschemas_any_array_item_type_2_item_data
                                    )
                                )

                                componentsschemas_any_array_item_type_2.append(
                                    componentsschemas_any_array_item_type_2_item
                                )

                            return componentsschemas_any_array_item_type_2
                        except:  # noqa: E722
                            pass
                        return cast(
                            Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str],
                            data,
                        )

                    componentsschemas_any_array_item = _parse_componentsschemas_any_array_item(
                        componentsschemas_any_array_item_data
                    )

                    componentsschemas_any_type_0.append(componentsschemas_any_array_item)

                return componentsschemas_any_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_any_type_1 = AnyObject.from_dict(data)

                return componentsschemas_any_type_1
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "AnyObject",
                    List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
                    Unset,
                    bool,
                    float,
                    int,
                    str,
                ],
                data,
            )

        incorrect_value = _parse_incorrect_value(d.pop("incorrect_value", UNSET))

        error_message = d.pop("error_message", UNSET)

        def _parse_fix_value(
            data: object,
        ) -> Union[
            "AnyObject",
            List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
            Unset,
            bool,
            float,
            int,
            str,
        ]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemas_any_type_0 = []
                _componentsschemas_any_type_0 = data
                for componentsschemas_any_array_item_data in _componentsschemas_any_type_0:

                    def _parse_componentsschemas_any_array_item(
                        data: object,
                    ) -> Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            componentsschemas_any_array_item_type_0 = AnyObject.from_dict(data)

                            return componentsschemas_any_array_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, list):
                                raise TypeError()
                            componentsschemas_any_array_item_type_2 = []
                            _componentsschemas_any_array_item_type_2 = data
                            for (
                                componentsschemas_any_array_item_type_2_item_data
                            ) in _componentsschemas_any_array_item_type_2:

                                def _parse_componentsschemas_any_array_item_type_2_item(
                                    data: object,
                                ) -> Union["AnyObject", bool, float, int, str]:
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        componentsschemas_any_array_item_type_2_item_type_0 = AnyObject.from_dict(data)

                                        return componentsschemas_any_array_item_type_2_item_type_0
                                    except:  # noqa: E722
                                        pass
                                    return cast(Union["AnyObject", bool, float, int, str], data)

                                componentsschemas_any_array_item_type_2_item = (
                                    _parse_componentsschemas_any_array_item_type_2_item(
                                        componentsschemas_any_array_item_type_2_item_data
                                    )
                                )

                                componentsschemas_any_array_item_type_2.append(
                                    componentsschemas_any_array_item_type_2_item
                                )

                            return componentsschemas_any_array_item_type_2
                        except:  # noqa: E722
                            pass
                        return cast(
                            Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str],
                            data,
                        )

                    componentsschemas_any_array_item = _parse_componentsschemas_any_array_item(
                        componentsschemas_any_array_item_data
                    )

                    componentsschemas_any_type_0.append(componentsschemas_any_array_item)

                return componentsschemas_any_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_any_type_1 = AnyObject.from_dict(data)

                return componentsschemas_any_type_1
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "AnyObject",
                    List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
                    Unset,
                    bool,
                    float,
                    int,
                    str,
                ],
                data,
            )

        fix_value = _parse_fix_value(d.pop("fix_value", UNSET))

        path = []
        _path = d.pop("path", UNSET)
        for path_item_data in _path or []:

            def _parse_path_item(
                data: object,
            ) -> Union[
                "AnyObject",
                List[Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]],
                bool,
                float,
                int,
                str,
            ]:
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    componentsschemas_any_type_0 = []
                    _componentsschemas_any_type_0 = data
                    for componentsschemas_any_array_item_data in _componentsschemas_any_type_0:

                        def _parse_componentsschemas_any_array_item(
                            data: object,
                        ) -> Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]:
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                componentsschemas_any_array_item_type_0 = AnyObject.from_dict(data)

                                return componentsschemas_any_array_item_type_0
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, list):
                                    raise TypeError()
                                componentsschemas_any_array_item_type_2 = []
                                _componentsschemas_any_array_item_type_2 = data
                                for (
                                    componentsschemas_any_array_item_type_2_item_data
                                ) in _componentsschemas_any_array_item_type_2:

                                    def _parse_componentsschemas_any_array_item_type_2_item(
                                        data: object,
                                    ) -> Union["AnyObject", bool, float, int, str]:
                                        try:
                                            if not isinstance(data, dict):
                                                raise TypeError()
                                            componentsschemas_any_array_item_type_2_item_type_0 = AnyObject.from_dict(
                                                data
                                            )

                                            return componentsschemas_any_array_item_type_2_item_type_0
                                        except:  # noqa: E722
                                            pass
                                        return cast(Union["AnyObject", bool, float, int, str], data)

                                    componentsschemas_any_array_item_type_2_item = (
                                        _parse_componentsschemas_any_array_item_type_2_item(
                                            componentsschemas_any_array_item_type_2_item_data
                                        )
                                    )

                                    componentsschemas_any_array_item_type_2.append(
                                        componentsschemas_any_array_item_type_2_item
                                    )

                                return componentsschemas_any_array_item_type_2
                            except:  # noqa: E722
                                pass
                            return cast(
                                Union[
                                    "AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str
                                ],
                                data,
                            )

                        componentsschemas_any_array_item = _parse_componentsschemas_any_array_item(
                            componentsschemas_any_array_item_data
                        )

                        componentsschemas_any_type_0.append(componentsschemas_any_array_item)

                    return componentsschemas_any_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_any_type_1 = AnyObject.from_dict(data)

                    return componentsschemas_any_type_1
                except:  # noqa: E722
                    pass
                return cast(
                    Union[
                        "AnyObject",
                        List[
                            Union["AnyObject", List[Union["AnyObject", bool, float, int, str]], bool, float, int, str]
                        ],
                        bool,
                        float,
                        int,
                        str,
                    ],
                    data,
                )

            path_item = _parse_path_item(path_item_data)

            path.append(path_item)

        reask = cls(
            incorrect_value=incorrect_value,
            error_message=error_message,
            fix_value=fix_value,
            path=path,
        )

        reask.additional_properties = d
        return reask

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.on_fail import OnFail


T = TypeVar("T", bound="SchemaElement")


@_attrs_define
class SchemaElement:
    """
    Attributes:
        type (str):
        name (Union[Unset, str]):
        description (Union[Unset, str]):
        strict (Union[Unset, bool]):
        date_format (Union[Unset, str]):
        time_format (Union[Unset, str]):
        on_fail (Union[Unset, str]):
        on_fails (Union[Unset, List['OnFail']]):
        model (Union[Unset, str]):
    """

    type: str
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    strict: Union[Unset, bool] = UNSET
    date_format: Union[Unset, str] = UNSET
    time_format: Union[Unset, str] = UNSET
    on_fail: Union[Unset, str] = UNSET
    on_fails: Union[Unset, List["OnFail"]] = UNSET
    model: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        name = self.name

        description = self.description

        strict = self.strict

        date_format = self.date_format

        time_format = self.time_format

        on_fail = self.on_fail

        on_fails: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.on_fails, Unset):
            on_fails = []
            for on_fails_item_data in self.on_fails:
                on_fails_item = on_fails_item_data.to_dict()
                on_fails.append(on_fails_item)

        model = self.model

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if strict is not UNSET:
            field_dict["strict"] = strict
        if date_format is not UNSET:
            field_dict["dateFormat"] = date_format
        if time_format is not UNSET:
            field_dict["timeFormat"] = time_format
        if on_fail is not UNSET:
            field_dict["onFail"] = on_fail
        if on_fails is not UNSET:
            field_dict["onFails"] = on_fails
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.on_fail import OnFail

        d = src_dict.copy()
        type = d.pop("type")

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        strict = d.pop("strict", UNSET)

        date_format = d.pop("dateFormat", UNSET)

        time_format = d.pop("timeFormat", UNSET)

        on_fail = d.pop("onFail", UNSET)

        on_fails = []
        _on_fails = d.pop("onFails", UNSET)
        for on_fails_item_data in _on_fails or []:
            on_fails_item = OnFail.from_dict(on_fails_item_data)

            on_fails.append(on_fails_item)

        model = d.pop("model", UNSET)

        schema_element = cls(
            type=type,
            name=name,
            description=description,
            strict=strict,
            date_format=date_format,
            time_format=time_format,
            on_fail=on_fail,
            on_fails=on_fails,
            model=model,
        )

        schema_element.additional_properties = d
        return schema_element

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document_metadata import DocumentMetadata
    from ..models.document_pages import DocumentPages


T = TypeVar("T", bound="Document")


@_attrs_define
class Document:
    """
    Attributes:
        id (Union[Unset, str]):
        pages (Union[Unset, DocumentPages]):
        metadata (Union[Unset, DocumentMetadata]):
    """

    id: Union[Unset, str] = UNSET
    pages: Union[Unset, "DocumentPages"] = UNSET
    metadata: Union[Unset, "DocumentMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        pages: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pages, Unset):
            pages = self.pages.to_dict()

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if pages is not UNSET:
            field_dict["pages"] = pages
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document_metadata import DocumentMetadata
        from ..models.document_pages import DocumentPages

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _pages = d.pop("pages", UNSET)
        pages: Union[Unset, DocumentPages]
        if isinstance(_pages, Unset):
            pages = UNSET
        else:
            pages = DocumentPages.from_dict(_pages)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, DocumentMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = DocumentMetadata.from_dict(_metadata)

        document = cls(
            id=id,
            pages=pages,
            metadata=metadata,
        )

        document.additional_properties = d
        return document

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ingestion_metadata import IngestionMetadata


T = TypeVar("T", bound="Ingestion")


@_attrs_define
class Ingestion:
    """
    Attributes:
        id (Union[Unset, str]):
        guard_id (Union[Unset, str]):
        embeddings (Union[Unset, List[List[float]]]):
        document_id (Union[Unset, str]):
        metadata (Union[Unset, IngestionMetadata]):
        validator_id (Union[Unset, str]):
        page_id (Union[Unset, str]):
        chunk_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    guard_id: Union[Unset, str] = UNSET
    embeddings: Union[Unset, List[List[float]]] = UNSET
    document_id: Union[Unset, str] = UNSET
    metadata: Union[Unset, "IngestionMetadata"] = UNSET
    validator_id: Union[Unset, str] = UNSET
    page_id: Union[Unset, str] = UNSET
    chunk_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        guard_id = self.guard_id

        embeddings: Union[Unset, List[List[float]]] = UNSET
        if not isinstance(self.embeddings, Unset):
            embeddings = []
            for embeddings_item_data in self.embeddings:
                embeddings_item = embeddings_item_data

                embeddings.append(embeddings_item)

        document_id = self.document_id

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        validator_id = self.validator_id

        page_id = self.page_id

        chunk_id = self.chunk_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if guard_id is not UNSET:
            field_dict["guardId"] = guard_id
        if embeddings is not UNSET:
            field_dict["embeddings"] = embeddings
        if document_id is not UNSET:
            field_dict["documentId"] = document_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if validator_id is not UNSET:
            field_dict["validatorId"] = validator_id
        if page_id is not UNSET:
            field_dict["pageId"] = page_id
        if chunk_id is not UNSET:
            field_dict["chunkId"] = chunk_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.ingestion_metadata import IngestionMetadata

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        guard_id = d.pop("guardId", UNSET)

        embeddings = []
        _embeddings = d.pop("embeddings", UNSET)
        for embeddings_item_data in _embeddings or []:
            embeddings_item = cast(List[float], embeddings_item_data)

            embeddings.append(embeddings_item)

        document_id = d.pop("documentId", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, IngestionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = IngestionMetadata.from_dict(_metadata)

        validator_id = d.pop("validatorId", UNSET)

        page_id = d.pop("pageId", UNSET)

        chunk_id = d.pop("chunkId", UNSET)

        ingestion = cls(
            id=id,
            guard_id=guard_id,
            embeddings=embeddings,
            document_id=document_id,
            metadata=metadata,
            validator_id=validator_id,
            page_id=page_id,
            chunk_id=chunk_id,
        )

        ingestion.additional_properties = d
        return ingestion

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_type_children import DataTypeChildren
    from ..models.schema_element import SchemaElement


T = TypeVar("T", bound="DataType")


@_attrs_define
class DataType:
    """
    Attributes:
        element (SchemaElement):
        children (Union[Unset, DataTypeChildren]):
        formatters (Union[Unset, List[str]]):
        plugins (Union[Unset, List[str]]):
    """

    element: "SchemaElement"
    children: Union[Unset, "DataTypeChildren"] = UNSET
    formatters: Union[Unset, List[str]] = UNSET
    plugins: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        element = self.element.to_dict()

        children: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.children, Unset):
            children = self.children.to_dict()

        formatters: Union[Unset, List[str]] = UNSET
        if not isinstance(self.formatters, Unset):
            formatters = self.formatters

        plugins: Union[Unset, List[str]] = UNSET
        if not isinstance(self.plugins, Unset):
            plugins = self.plugins

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "element": element,
            }
        )
        if children is not UNSET:
            field_dict["children"] = children
        if formatters is not UNSET:
            field_dict["formatters"] = formatters
        if plugins is not UNSET:
            field_dict["plugins"] = plugins

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_type_children import DataTypeChildren
        from ..models.schema_element import SchemaElement

        d = src_dict.copy()
        element = SchemaElement.from_dict(d.pop("element"))

        _children = d.pop("children", UNSET)
        children: Union[Unset, DataTypeChildren]
        if isinstance(_children, Unset):
            children = UNSET
        else:
            children = DataTypeChildren.from_dict(_children)

        formatters = cast(List[str], d.pop("formatters", UNSET))

        plugins = cast(List[str], d.pop("plugins", UNSET))

        data_type = cls(
            element=element,
            children=children,
            formatters=formatters,
            plugins=plugins,
        )

        data_type.additional_properties = d
        return data_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.on_fail_options import OnFailOptions

T = TypeVar("T", bound="OnFail")


@_attrs_define
class OnFail:
    """
    Attributes:
        validator_tag (str):
        method (OnFailOptions):
    """

    validator_tag: str
    method: OnFailOptions
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        validator_tag = self.validator_tag

        method = self.method.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "validatorTag": validator_tag,
                "method": method,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        validator_tag = d.pop("validatorTag")

        method = OnFailOptions(d.pop("method"))

        on_fail = cls(
            validator_tag=validator_tag,
            method=method,
        )

        on_fail.additional_properties = d
        return on_fail

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.validate_payload_llm_api import ValidatePayloadLlmApi
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.validate_payload_prompt_params import ValidatePayloadPromptParams


T = TypeVar("T", bound="ValidatePayload")


@_attrs_define
class ValidatePayload:
    """
    Attributes:
        llm_output (Union[Unset, str]): The LLM output as a string or the input prompts for the LLM Example: stubbed llm
            output.
        num_reasks (Union[Unset, int]): An override for the number of re-asks to perform
        prompt_params (Union[Unset, ValidatePayloadPromptParams]): additional params for llm prompts
        llm_api (Union[Unset, ValidatePayloadLlmApi]):
    """

    llm_output: Union[Unset, str] = UNSET
    num_reasks: Union[Unset, int] = UNSET
    prompt_params: Union[Unset, "ValidatePayloadPromptParams"] = UNSET
    llm_api: Union[Unset, ValidatePayloadLlmApi] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        llm_output = self.llm_output

        num_reasks = self.num_reasks

        prompt_params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.prompt_params, Unset):
            prompt_params = self.prompt_params.to_dict()

        llm_api: Union[Unset, str] = UNSET
        if not isinstance(self.llm_api, Unset):
            llm_api = self.llm_api.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if llm_output is not UNSET:
            field_dict["llmOutput"] = llm_output
        if num_reasks is not UNSET:
            field_dict["numReasks"] = num_reasks
        if prompt_params is not UNSET:
            field_dict["promptParams"] = prompt_params
        if llm_api is not UNSET:
            field_dict["llmApi"] = llm_api

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validate_payload_prompt_params import ValidatePayloadPromptParams

        d = src_dict.copy()
        llm_output = d.pop("llmOutput", UNSET)

        num_reasks = d.pop("numReasks", UNSET)

        _prompt_params = d.pop("promptParams", UNSET)
        prompt_params: Union[Unset, ValidatePayloadPromptParams]
        if isinstance(_prompt_params, Unset):
            prompt_params = UNSET
        else:
            prompt_params = ValidatePayloadPromptParams.from_dict(_prompt_params)

        _llm_api = d.pop("llmApi", UNSET)
        llm_api: Union[Unset, ValidatePayloadLlmApi]
        if isinstance(_llm_api, Unset):
            llm_api = UNSET
        else:
            llm_api = ValidatePayloadLlmApi(_llm_api)

        validate_payload = cls(
            llm_output=llm_output,
            num_reasks=num_reasks,
            prompt_params=prompt_params,
            llm_api=llm_api,
        )

        validate_payload.additional_properties = d
        return validate_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.validator_metrics import ValidatorMetrics


T = TypeVar("T", bound="GuardRunStep")


@_attrs_define
class GuardRunStep:
    """
    Attributes:
        step_number (float):
        validator_metrics (List['ValidatorMetrics']):
    """

    step_number: float
    validator_metrics: List["ValidatorMetrics"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        step_number = self.step_number

        validator_metrics = []
        for validator_metrics_item_data in self.validator_metrics:
            validator_metrics_item = validator_metrics_item_data.to_dict()
            validator_metrics.append(validator_metrics_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "stepNumber": step_number,
                "validatorMetrics": validator_metrics,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validator_metrics import ValidatorMetrics

        d = src_dict.copy()
        step_number = d.pop("stepNumber")

        validator_metrics = []
        _validator_metrics = d.pop("validatorMetrics")
        for validator_metrics_item_data in _validator_metrics:
            validator_metrics_item = ValidatorMetrics.from_dict(validator_metrics_item_data)

            validator_metrics.append(validator_metrics_item)

        guard_run_step = cls(
            step_number=step_number,
            validator_metrics=validator_metrics,
        )

        guard_run_step.additional_properties = d
        return guard_run_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.on_fail_options import OnFailOptions

if TYPE_CHECKING:
    from ..models.validator_run_metric_details_parameters import ValidatorRunMetricDetailsParameters


T = TypeVar("T", bound="ValidatorRunMetricDetails")


@_attrs_define
class ValidatorRunMetricDetails:
    """
    Attributes:
        step_number (int):
        result_type (str):
        fail_action (OnFailOptions):
        parameters (ValidatorRunMetricDetailsParameters):
    """

    step_number: int
    result_type: str
    fail_action: OnFailOptions
    parameters: "ValidatorRunMetricDetailsParameters"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        step_number = self.step_number

        result_type = self.result_type

        fail_action = self.fail_action.value

        parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "stepNumber": step_number,
                "resultType": result_type,
                "failAction": fail_action,
                "parameters": parameters,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validator_run_metric_details_parameters import ValidatorRunMetricDetailsParameters

        d = src_dict.copy()
        step_number = d.pop("stepNumber")

        result_type = d.pop("resultType")

        fail_action = OnFailOptions(d.pop("failAction"))

        parameters = ValidatorRunMetricDetailsParameters.from_dict(d.pop("parameters"))

        validator_run_metric_details = cls(
            step_number=step_number,
            result_type=result_type,
            fail_action=fail_action,
            parameters=parameters,
        )

        validator_run_metric_details.additional_properties = d
        return validator_run_metric_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ValidatePayloadLlmApi(str, Enum):
    OPENAI_CHATCOMPLETION_ACREATE = "openai.ChatCompletion.acreate"
    OPENAI_CHATCOMPLETION_CREATE = "openai.ChatCompletion.create"
    OPENAI_COMPLETION_ACREATE = "openai.Completion.acreate"
    OPENAI_COMPLETION_CREATE = "openai.Completion.create"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.validator_metrics import ValidatorMetrics
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    start_time: Union[Unset, datetime.datetime] = UNSET,
    end_time: Union[Unset, datetime.datetime] = UNSET,
    page: Union[Unset, int] = 0,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_start_time: Union[Unset, str] = UNSET
    if not isinstance(start_time, Unset):
        json_start_time = start_time.isoformat()
    params["startTime"] = json_start_time

    json_end_time: Union[Unset, str] = UNSET
    if not isinstance(end_time, Unset):
        json_end_time = end_time.isoformat()
    params["endTime"] = json_end_time

    params["page"] = page

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/validators/metrics",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[List["ValidatorMetrics"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ValidatorMetrics.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[List["ValidatorMetrics"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    start_time: Union[Unset, datetime.datetime] = UNSET,
    end_time: Union[Unset, datetime.datetime] = UNSET,
    page: Union[Unset, int] = 0,
) -> Response[List["ValidatorMetrics"]]:
    """Returns an array of ValidatorMetrics for a given time range.  If startTime or endTime are ommitted,
    a five minute duration is used by default.  Default page size is 10, but this page size applied to
    guard steps not validator metrics. Have to think this one out a little more.

    Args:
        start_time (Union[Unset, datetime.datetime]):
        end_time (Union[Unset, datetime.datetime]):
        page (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['ValidatorMetrics']]
    """

    kwargs = _get_kwargs(
        start_time=start_time,
        end_time=end_time,
        page=page,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    start_time: Union[Unset, datetime.datetime] = UNSET,
    end_time: Union[Unset, datetime.datetime] = UNSET,
    page: Union[Unset, int] = 0,
) -> Optional[List["ValidatorMetrics"]]:
    """Returns an array of ValidatorMetrics for a given time range.  If startTime or endTime are ommitted,
    a five minute duration is used by default.  Default page size is 10, but this page size applied to
    guard steps not validator metrics. Have to think this one out a little more.

    Args:
        start_time (Union[Unset, datetime.datetime]):
        end_time (Union[Unset, datetime.datetime]):
        page (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['ValidatorMetrics']
    """

    return sync_detailed(
        client=client,
        start_time=start_time,
        end_time=end_time,
        page=page,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    start_time: Union[Unset, datetime.datetime] = UNSET,
    end_time: Union[Unset, datetime.datetime] = UNSET,
    page: Union[Unset, int] = 0,
) -> Response[List["ValidatorMetrics"]]:
    """Returns an array of ValidatorMetrics for a given time range.  If startTime or endTime are ommitted,
    a five minute duration is used by default.  Default page size is 10, but this page size applied to
    guard steps not validator metrics. Have to think this one out a little more.

    Args:
        start_time (Union[Unset, datetime.datetime]):
        end_time (Union[Unset, datetime.datetime]):
        page (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['ValidatorMetrics']]
    """

    kwargs = _get_kwargs(
        start_time=start_time,
        end_time=end_time,
        page=page,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    start_time: Union[Unset, datetime.datetime] = UNSET,
    end_time: Union[Unset, datetime.datetime] = UNSET,
    page: Union[Unset, int] = 0,
) -> Optional[List["ValidatorMetrics"]]:
    """Returns an array of ValidatorMetrics for a given time range.  If startTime or endTime are ommitted,
    a five minute duration is used by default.  Default page size is 10, but this page size applied to
    guard steps not validator metrics. Have to think this one out a little more.

    Args:
        start_time (Union[Unset, datetime.datetime]):
        end_time (Union[Unset, datetime.datetime]):
        page (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['ValidatorMetrics']
    """

    return (
        await asyncio_detailed(
            client=client,
            start_time=start_time,
            end_time=end_time,
            page=page,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status import Status

T = TypeVar("T", bound="GuardMetrics")


@_attrs_define
class GuardMetrics:
    """
    Attributes:
        request_id (str): The id of the request this metric entry is for (i.e. the specific call to
            /guards/{guardName}/validate)
        guard_name (str): The name of the Guard
        timestamp (datetime.datetime): The start time of the span the metrics were captured within (ISO 8601 format; See
            https://www.rfc-editor.org/rfc/rfc3339#section-5.6)
        run_duration (float): The total span duration of the guard run in seconds
        status (Status):
        reasks (float): The number of times the LLM was reasked
        tokens (float): The number of tokens used
        raw_llm_ouput (str): The unaltered output from the LLM
        validated_output (str): The potentially altered output from validation
    """

    request_id: str
    guard_name: str
    timestamp: datetime.datetime
    run_duration: float
    status: Status
    reasks: float
    tokens: float
    raw_llm_ouput: str
    validated_output: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_id = self.request_id

        guard_name = self.guard_name

        timestamp = self.timestamp.isoformat()

        run_duration = self.run_duration

        status = self.status.value

        reasks = self.reasks

        tokens = self.tokens

        raw_llm_ouput = self.raw_llm_ouput

        validated_output = self.validated_output

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestId": request_id,
                "guardName": guard_name,
                "timestamp": timestamp,
                "runDuration": run_duration,
                "status": status,
                "reasks": reasks,
                "tokens": tokens,
                "rawLlmOuput": raw_llm_ouput,
                "validatedOutput": validated_output,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_id = d.pop("requestId")

        guard_name = d.pop("guardName")

        timestamp = isoparse(d.pop("timestamp"))

        run_duration = d.pop("runDuration")

        status = Status(d.pop("status"))

        reasks = d.pop("reasks")

        tokens = d.pop("tokens")

        raw_llm_ouput = d.pop("rawLlmOuput")

        validated_output = d.pop("validatedOutput")

        guard_metrics = cls(
            request_id=request_id,
            guard_name=guard_name,
            timestamp=timestamp,
            run_duration=run_duration,
            status=status,
            reasks=reasks,
            tokens=tokens,
            raw_llm_ouput=raw_llm_ouput,
            validated_output=validated_output,
        )

        guard_metrics.additional_properties = d
        return guard_metrics

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.data_type import DataType
    from ..models.json_schema import JsonSchema


T = TypeVar("T", bound="Schema")


@_attrs_define
class Schema:
    """
    Attributes:
        schema (Union['DataType', 'JsonSchema']):
    """

    schema: Union["DataType", "JsonSchema"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.json_schema import JsonSchema

        schema: Dict[str, Any]
        if isinstance(self.schema, JsonSchema):
            schema = self.schema.to_dict()
        else:
            schema = self.schema.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "schema": schema,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_type import DataType
        from ..models.json_schema import JsonSchema

        d = src_dict.copy()

        def _parse_schema(data: object) -> Union["DataType", "JsonSchema"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                schema_type_0 = JsonSchema.from_dict(data)

                return schema_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            schema_type_1 = DataType.from_dict(data)

            return schema_type_1

        schema = _parse_schema(d.pop("schema"))

        schema = cls(
            schema=schema,
        )

        schema.additional_properties = d
        return schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document import Document


T = TypeVar("T", bound="IngestionPayload")


@_attrs_define
class IngestionPayload:
    """
    Attributes:
        articles (Union[Unset, Document]):
        guard_id (Union[Unset, str]):
        validator_id (Union[Unset, str]):
        chunk_id (Union[Unset, str]):
    """

    articles: Union[Unset, "Document"] = UNSET
    guard_id: Union[Unset, str] = UNSET
    validator_id: Union[Unset, str] = UNSET
    chunk_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        articles: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.articles, Unset):
            articles = self.articles.to_dict()

        guard_id = self.guard_id

        validator_id = self.validator_id

        chunk_id = self.chunk_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if articles is not UNSET:
            field_dict["articles"] = articles
        if guard_id is not UNSET:
            field_dict["guardId"] = guard_id
        if validator_id is not UNSET:
            field_dict["validatorId"] = validator_id
        if chunk_id is not UNSET:
            field_dict["chunkId"] = chunk_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document import Document

        d = src_dict.copy()
        _articles = d.pop("articles", UNSET)
        articles: Union[Unset, Document]
        if isinstance(_articles, Unset):
            articles = UNSET
        else:
            articles = Document.from_dict(_articles)

        guard_id = d.pop("guardId", UNSET)

        validator_id = d.pop("validatorId", UNSET)

        chunk_id = d.pop("chunkId", UNSET)

        ingestion_payload = cls(
            articles=articles,
            guard_id=guard_id,
            validator_id=validator_id,
            chunk_id=chunk_id,
        )

        ingestion_payload.additional_properties = d
        return ingestion_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OnFailOptions(str, Enum):
    EXCEPTION = "exception"
    FILTER = "filter"
    FIX = "fix"
    FIX_REASK = "fix_reask"
    NOOP = "noop"
    REASK = "reask"
    REFRAIN = "refrain"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status import Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="ValidatorMetrics")


@_attrs_define
class ValidatorMetrics:
    """
    Attributes:
        instance_id (str):
        name (str):
        request_id (str):
        guard_name (str):
        start_time (datetime.datetime):
        end_time (datetime.datetime):
        duration_in_millis (float):
        input_ (str):
        output (str):
        status (Status):
        tokens (Union[Unset, float]): The number of tokens consumed if the validator calls an LLM. Only applies to
            validators that call LLMs.
    """

    instance_id: str
    name: str
    request_id: str
    guard_name: str
    start_time: datetime.datetime
    end_time: datetime.datetime
    duration_in_millis: float
    input_: str
    output: str
    status: Status
    tokens: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instance_id = self.instance_id

        name = self.name

        request_id = self.request_id

        guard_name = self.guard_name

        start_time = self.start_time.isoformat()

        end_time = self.end_time.isoformat()

        duration_in_millis = self.duration_in_millis

        input_ = self.input_

        output = self.output

        status = self.status.value

        tokens = self.tokens

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "instanceId": instance_id,
                "name": name,
                "requestId": request_id,
                "guardName": guard_name,
                "startTime": start_time,
                "endTime": end_time,
                "durationInMillis": duration_in_millis,
                "input": input_,
                "output": output,
                "status": status,
            }
        )
        if tokens is not UNSET:
            field_dict["tokens"] = tokens

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        instance_id = d.pop("instanceId")

        name = d.pop("name")

        request_id = d.pop("requestId")

        guard_name = d.pop("guardName")

        start_time = isoparse(d.pop("startTime"))

        end_time = isoparse(d.pop("endTime"))

        duration_in_millis = d.pop("durationInMillis")

        input_ = d.pop("input")

        output = d.pop("output")

        status = Status(d.pop("status"))

        tokens = d.pop("tokens", UNSET)

        validator_metrics = cls(
            instance_id=instance_id,
            name=name,
            request_id=request_id,
            guard_name=guard_name,
            start_time=start_time,
            end_time=end_time,
            duration_in_millis=duration_in_millis,
            input_=input_,
            output=output,
            status=status,
            tokens=tokens,
        )

        validator_metrics.additional_properties = d
        return validator_metrics

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.guard_run_step import GuardRunStep


T = TypeVar("T", bound="GuardRunMetricDetails")


@_attrs_define
class GuardRunMetricDetails:
    """
    Attributes:
        steps (List['GuardRunStep']):
        prompt (Union[Unset, str]):
        instructions (Union[Unset, str]):
    """

    steps: List["GuardRunStep"]
    prompt: Union[Unset, str] = UNSET
    instructions: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        steps = []
        for steps_item_data in self.steps:
            steps_item = steps_item_data.to_dict()
            steps.append(steps_item)

        prompt = self.prompt

        instructions = self.instructions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "steps": steps,
            }
        )
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if instructions is not UNSET:
            field_dict["instructions"] = instructions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.guard_run_step import GuardRunStep

        d = src_dict.copy()
        steps = []
        _steps = d.pop("steps")
        for steps_item_data in _steps:
            steps_item = GuardRunStep.from_dict(steps_item_data)

            steps.append(steps_item)

        prompt = d.pop("prompt", UNSET)

        instructions = d.pop("instructions", UNSET)

        guard_run_metric_details = cls(
            steps=steps,
            prompt=prompt,
            instructions=instructions,
        )

        guard_run_metric_details.additional_properties = d
        return guard_run_metric_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.data_type import DataType


T = TypeVar("T", bound="JsonSchema")


@_attrs_define
class JsonSchema:
    """ """

    additional_properties: Dict[str, "DataType"] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_type import DataType

        d = src_dict.copy()
        json_schema = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = DataType.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        json_schema.additional_properties = additional_properties
        return json_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "DataType":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "DataType") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .any_object import AnyObject
from .data_type import DataType
from .data_type_children import DataTypeChildren
from .document import Document
from .document_metadata import DocumentMetadata
from .document_pages import DocumentPages
from .guard import Guard
from .guard_guard_config import GuardGuardConfig
from .guard_metrics import GuardMetrics
from .guard_run_metric_details import GuardRunMetricDetails
from .guard_run_metrics import GuardRunMetrics
from .guard_run_step import GuardRunStep
from .health_check import HealthCheck
from .history import History
from .history_event import HistoryEvent
from .history_event_prompt import HistoryEventPrompt
from .history_event_reasks_item import HistoryEventReasksItem
from .http_error import HttpError
from .http_error_fields import HttpErrorFields
from .ingestion import Ingestion
from .ingestion_metadata import IngestionMetadata
from .ingestion_payload import IngestionPayload
from .json_schema import JsonSchema
from .on_fail import OnFail
from .on_fail_options import OnFailOptions
from .projection import Projection
from .rail_spec import RailSpec
from .rail_spec_version import RailSpecVersion
from .reask import Reask
from .schema import Schema
from .schema_element import SchemaElement
from .status import Status
from .validate_payload import ValidatePayload
from .validate_payload_llm_api import ValidatePayloadLlmApi
from .validate_payload_prompt_params import ValidatePayloadPromptParams
from .validation_output import ValidationOutput
from .validator_metrics import ValidatorMetrics
from .validator_run_metric_details import ValidatorRunMetricDetails
from .validator_run_metric_details_parameters import ValidatorRunMetricDetailsParameters
from .validator_run_metrics import ValidatorRunMetrics

__all__ = (
    "AnyObject",
    "DataType",
    "DataTypeChildren",
    "Document",
    "DocumentMetadata",
    "DocumentPages",
    "Guard",
    "GuardGuardConfig",
    "GuardMetrics",
    "GuardRunMetricDetails",
    "GuardRunMetrics",
    "GuardRunStep",
    "HealthCheck",
    "History",
    "HistoryEvent",
    "HistoryEventPrompt",
    "HistoryEventReasksItem",
    "HttpError",
    "HttpErrorFields",
    "Ingestion",
    "IngestionMetadata",
    "IngestionPayload",
    "JsonSchema",
    "OnFail",
    "OnFailOptions",
    "Projection",
    "RailSpec",
    "RailSpecVersion",
    "Reask",
    "Schema",
    "SchemaElement",
    "Status",
    "ValidatePayload",
    "ValidatePayloadLlmApi",
    "ValidatePayloadPromptParams",
    "ValidationOutput",
    "ValidatorMetrics",
    "ValidatorRunMetricDetails",
    "ValidatorRunMetricDetailsParameters",
    "ValidatorRunMetrics",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.any_object import AnyObject
    from ..models.history import History


T = TypeVar("T", bound="ValidationOutput")


@_attrs_define
class ValidationOutput:
    """
    Attributes:
        result (bool): Whether the validation passed or failed
        validated_output (Union['AnyObject', Unset, str]):
        session_history (Union[Unset, List['History']]):
        raw_llm_response (Union[Unset, str]):
    """

    result: bool
    validated_output: Union["AnyObject", Unset, str] = UNSET
    session_history: Union[Unset, List["History"]] = UNSET
    raw_llm_response: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.any_object import AnyObject

        result = self.result

        validated_output: Union[Dict[str, Any], Unset, str]
        if isinstance(self.validated_output, Unset):
            validated_output = UNSET
        elif isinstance(self.validated_output, AnyObject):
            validated_output = self.validated_output.to_dict()
        else:
            validated_output = self.validated_output

        session_history: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.session_history, Unset):
            session_history = []
            for session_history_item_data in self.session_history:
                session_history_item = session_history_item_data.to_dict()
                session_history.append(session_history_item)

        raw_llm_response = self.raw_llm_response

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "result": result,
            }
        )
        if validated_output is not UNSET:
            field_dict["validatedOutput"] = validated_output
        if session_history is not UNSET:
            field_dict["sessionHistory"] = session_history
        if raw_llm_response is not UNSET:
            field_dict["rawLlmResponse"] = raw_llm_response

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.any_object import AnyObject
        from ..models.history import History

        d = src_dict.copy()
        result = d.pop("result")

        def _parse_validated_output(data: object) -> Union["AnyObject", Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                validated_output_type_0 = AnyObject.from_dict(data)

                return validated_output_type_0
            except:  # noqa: E722
                pass
            return cast(Union["AnyObject", Unset, str], data)

        validated_output = _parse_validated_output(d.pop("validatedOutput", UNSET))

        session_history = []
        _session_history = d.pop("sessionHistory", UNSET)
        for session_history_item_data in _session_history or []:
            session_history_item = History.from_dict(session_history_item_data)

            session_history.append(session_history_item)

        raw_llm_response = d.pop("rawLlmResponse", UNSET)

        validation_output = cls(
            result=result,
            validated_output=validated_output,
            session_history=session_history,
            raw_llm_response=raw_llm_response,
        )

        validation_output.additional_properties = d
        return validation_output

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pathlib

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="guardrails-api-client",
    version="0.1.1",
    description="A client library for accessing Guardrails API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    python_requires=">=3.8, <4",
    install_requires=["httpx >= 0.20.0, < 0.28.0", "attrs >= 21.3.0", "python-dateutil >= 2.8.0, < 3"],
    package_data={"guardrails_api_client": ["py.typed"]},
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.any_object import AnyObject
    from ..models.history_event_prompt import HistoryEventPrompt
    from ..models.history_event_reasks_item import HistoryEventReasksItem


T = TypeVar("T", bound="HistoryEvent")


@_attrs_define
class HistoryEvent:
    """
    Attributes:
        instructions (Union[Unset, str]):
        output (Union[Unset, str]):
        parsed_output (Union['AnyObject', Unset, str]):
        prompt (Union[Unset, HistoryEventPrompt]):
        reasks (Union[Unset, List['HistoryEventReasksItem']]):
        validated_output (Union['AnyObject', Unset, str]):
    """

    instructions: Union[Unset, str] = UNSET
    output: Union[Unset, str] = UNSET
    parsed_output: Union["AnyObject", Unset, str] = UNSET
    prompt: Union[Unset, "HistoryEventPrompt"] = UNSET
    reasks: Union[Unset, List["HistoryEventReasksItem"]] = UNSET
    validated_output: Union["AnyObject", Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.any_object import AnyObject

        instructions = self.instructions

        output = self.output

        parsed_output: Union[Dict[str, Any], Unset, str]
        if isinstance(self.parsed_output, Unset):
            parsed_output = UNSET
        elif isinstance(self.parsed_output, AnyObject):
            parsed_output = self.parsed_output.to_dict()
        else:
            parsed_output = self.parsed_output

        prompt: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.prompt, Unset):
            prompt = self.prompt.to_dict()

        reasks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.reasks, Unset):
            reasks = []
            for reasks_item_data in self.reasks:
                reasks_item = reasks_item_data.to_dict()
                reasks.append(reasks_item)

        validated_output: Union[Dict[str, Any], Unset, str]
        if isinstance(self.validated_output, Unset):
            validated_output = UNSET
        elif isinstance(self.validated_output, AnyObject):
            validated_output = self.validated_output.to_dict()
        else:
            validated_output = self.validated_output

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instructions is not UNSET:
            field_dict["instructions"] = instructions
        if output is not UNSET:
            field_dict["output"] = output
        if parsed_output is not UNSET:
            field_dict["parsedOutput"] = parsed_output
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if reasks is not UNSET:
            field_dict["reasks"] = reasks
        if validated_output is not UNSET:
            field_dict["validatedOutput"] = validated_output

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.any_object import AnyObject
        from ..models.history_event_prompt import HistoryEventPrompt
        from ..models.history_event_reasks_item import HistoryEventReasksItem

        d = src_dict.copy()
        instructions = d.pop("instructions", UNSET)

        output = d.pop("output", UNSET)

        def _parse_parsed_output(data: object) -> Union["AnyObject", Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parsed_output_type_0 = AnyObject.from_dict(data)

                return parsed_output_type_0
            except:  # noqa: E722
                pass
            return cast(Union["AnyObject", Unset, str], data)

        parsed_output = _parse_parsed_output(d.pop("parsedOutput", UNSET))

        _prompt = d.pop("prompt", UNSET)
        prompt: Union[Unset, HistoryEventPrompt]
        if isinstance(_prompt, Unset):
            prompt = UNSET
        else:
            prompt = HistoryEventPrompt.from_dict(_prompt)

        reasks = []
        _reasks = d.pop("reasks", UNSET)
        for reasks_item_data in _reasks or []:
            reasks_item = HistoryEventReasksItem.from_dict(reasks_item_data)

            reasks.append(reasks_item)

        def _parse_validated_output(data: object) -> Union["AnyObject", Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                validated_output_type_0 = AnyObject.from_dict(data)

                return validated_output_type_0
            except:  # noqa: E722
                pass
            return cast(Union["AnyObject", Unset, str], data)

        validated_output = _parse_validated_output(d.pop("validatedOutput", UNSET))

        history_event = cls(
            instructions=instructions,
            output=output,
            parsed_output=parsed_output,
            prompt=prompt,
            reasks=reasks,
            validated_output=validated_output,
        )

        history_event.additional_properties = d
        return history_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
